<?php
namespace PME\Controllers\Medicao;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class EfetivosRelatoriosController extends \Zeedhi\Framework\Controller\Report {
    
    protected $reportService;
    protected $birtStrategy;
    protected $utilities;
    protected $Lista;
    protected $connection;

    public function __construct($reportService, $birtStrategy, $utilities, $Lista, $connection) {
        $this->birtStrategy  = $birtStrategy;
        $this->utilities     = $utilities;
        $this->Lista         = $Lista;
        $this->connection    = $connection;
        
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'PME01400' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "DTPERIINI"    => "P_DTPERIINI",
                "DTPERIFIN"    => "P_DTPERIFIN",
                "NMFILCENTPME" => "P_NMFILCENTPME",
                "CDFILCENTPME" => "P_CDFILCENTPME",
                "CDTIPOCARD"   => "P_CDTIPOCARD",
                "DTINI"        => "P_DTINICIAL",
                "DTFIN"        => "P_DTFINAL",
                "NRLISTA"      => "NRLISTA",
                "NRORG"        => "NRORG"
            )
        ),
        'PME01401' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "DTPERIINI"    => "P_DTPERIINI",
                "DTPERIFIN"    => "P_DTPERIFIN",
                "NMFILCENTPME" => "P_NMFILCENTPME",
                "CDFILCENTPME" => "P_CDFILCENTPME",
                "CDTIPOCARD"   => "P_CDTIPOCARD",
                "DTINI"        => "P_DTINICIAL",
                "DTFIN"        => "P_DTFINAL",
                "NRLISTA"      => "NRLISTA",
                "NRORG"        => "NRORG"
            )
        ),
        'PME01402' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "DTPERIINI"    => "P_DTPERIINI",
                "DTPERIFIN"    => "P_DTPERIFIN",
                "NMFILCENTPME" => "P_NMFILCENTPME",
                "CDFILCENTPME" => "P_CDFILCENTPME",
                "CDTIPOCARD"   => "P_CDTIPOCARD",
                "DTINI"        => "P_DTINICIAL",
                "DTFIN"        => "P_DTFINAL",
                "NRLISTA"      => "NRLISTA",
                "NRORG"        => "NRORG"
            )
        ),
        'PME01403' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "DTPERIINI"    => "P_DTPERIINI",
                "DTPERIFIN"    => "P_DTPERIFIN",
                "NMFILCENTPME" => "P_NMFILCENTPME",
                "CDFILCENTPME" => "P_CDFILCENTPME",
                "CDTIPOCARD"   => "P_CDTIPOCARD",
                "DTINI"        => "P_DTINICIAL",
                "DTFIN"        => "P_DTFINAL",
                "NRLISTA"      => "NRLISTA",
                "NRORG"        => "NRORG"
            )
        ),
        'PME01405' => array (
            'strategy' => 'BIRT',
            'parameterMapping' => array (
                "DTPERIINI"    => "P_DTPERIINI",
                "DTPERIFIN"    => "P_DTPERIFIN",
                "NMFILCENTPME" => "P_NMFILCENTPME",
                "CDFILCENTPME" => "P_CDFILCENTPME",
                "CDTIPOCARD"   => "P_CDTIPOCARD",
                "DTINI"        => "P_DTINICIAL",
                "DTFIN"        => "P_DTFINAL",
                "NRLISTA"      => "NRLISTA",
                "NRORG"        => "NRORG"
            )
        ),
    );
	
    private function returnDadosLista($row, $unidade, $servico, $cardapio, $nrorg) {
        $sql = "SELECT DISTINCT S.CDFILCENTPME, F.CDFILIAL, FI.NMFILIAL, 
                       S.CDSERVFILI, SE.NMSERVICO, S.CDTIPOCARD,
                       (F.CDFILIAL || S.CDSERVFILI)AS UNIDSERV, SE.NRORG 
                  FROM SERVTIPOCARD S, FILITIPOCARD F, SERVICO SE, 
                       FILIAL FI 
                 WHERE S.CDFILCENTPME = :CDFILCENTPME 
                   AND S.CDFILCENTPME = F.CDFILCENTPME 
                   AND (INSTR(:CDTIPOCARD, S.CDTIPOCARD ) <> 0 OR :CDTIPOCARD = 'T')
                   AND S.CDTIPOCARD = F.CDTIPOCARD 
                   AND SE.CDSERVFILI = S.CDSERVFILI 
                   AND F.CDFILIAL = SE.CDFILIAL 
                   AND (INSTR( :CDFILIAL, SE.CDFILIAL ) <> 0 OR :CDFILIAL = 'T') 
                   AND (INSTR( :CDSERVFILI, SE.CDSERVFILI ) <> 0 OR :CDSERVFILI = 'T') 
                   AND (F.CDFILIAL = FI.CDFILIAL) 
                 ORDER BY F.CDFILIAL, S.CDSERVFILI, UNIDSERV";
        
        $params = array(
          'CDSERVFILI'   => $servico,
          'CDTIPOCARD'   => $cardapio,
          'CDFILCENTPME' => $row['CDFILCENTPME'],
          'CDFILIAL'     => $unidade
        );
        
        $stmt = $this->connection->prepare($sql);
        $stmt->execute($params);
        $result = $stmt->fetchAll();
        
        $unidServ = [];
        foreach($result as $row) {
            array_push($unidServ, $row['UNIDSERV']);
        }

        $concatUnidServ = join("_", $unidServ);
        $nrlista = $this->Lista->geraLista($concatUnidServ, $nrorg);
        return $nrlista;
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $nrorg = $this->utilities->getNrOrg();
        $row['NRORG'] = $nrorg;
        $countFilial = count($row['CDFILIAL']);
        $countServ = count($row['CDSERVFILI']);
        if ($row['REPORT']['CD_REPORT_CODE'] === 'PME01405') {
            $countCardapio = count($row['CDTIPOCARD_MULT']);
        }
        
        if ($countFilial > 0) $unidade = join("_", $row['CDFILIAL']);
        else $unidade = 'T';
        
        if ($countServ > 0) $servico = join("_", $row['CDSERVFILI']);
        else $servico = 'T';
        
        if (isset($countCardapio) === true) {
            if ($countCardapio > 0) $cardapio = join("_", $row['CDTIPOCARD_MULT']);
            else $cardapio = 'T';
        } else $cardapio = $row['CDTIPOCARD'];
        
        $row['CDTIPOCARD'] = $cardapio;
        $row['NRLISTA'] = $this->returnDadosLista($row, $unidade, $servico, $cardapio, $nrorg);
        $row['DTPERIINI'] = $row['DTINI'];
        $row['DTPERIFIN'] = $row['DTFIN'];

        $this->birtStrategy->setReportLocale($this->utilities->getLanguage());
        $this->birtStrategy->setReportFormat(strtolower($row['reportFormat']));
    }
}