import {
  ModalService, Modal, Form, Select, IButtonEvent, Grid, GridEditable, Card,
  LoadingService,
} from '@zeedhi/common';
import {
  Loader, Metadata, IEventParam, Http, DatasourceFactory, IDictionary,
} from '@zeedhi/core';
  
export class Pla01300PercentualReceitaController {
  private appController = Loader.getInstance('AppController');
  
  private utilities = Loader.getInstance('Utilities');
  
  private utilitiesPLA = Loader.getInstance('UtilitiesPLA');
  
  private prato = Loader.getInstance('Pla01300PratoController');
  
  private gridPlateIsVisible = false;
  
  public modalPercentualReceita!: Modal;
  
  public gridEditableController = Loader.getInstance('GridEditableController');
  
  private shouldLoadPratos: boolean = false;
  
  private async createModalPercentualReceita() {
    this.modalPercentualReceita = await ModalService.createFromJsonPost('',
      { jsonPath: 'planejamento/prato/modals/pla01303_percentual_receitas_servico' });
  }
  
  public closeModalPercentualReceita() {
    this.modalPercentualReceita.hide();
    this.modalPercentualReceita.destroy();
    this.modalPercentualReceita = this.utilities.modalEmpty;
  }
  
  public openModalPercentualReceita() {
    if (!this.modalPercentualReceita) {
      this.createModalPercentualReceita().then(() => {
        this.modalPercentualReceita.show();
      });
    } else {
      this.modalPercentualReceita.show();
    }
  }

  public fullscreen({ component }: IButtonEvent) {
    this.modalPercentualReceita.fullscreen = !this.modalPercentualReceita.fullscreen;
    component.iconName = (this.modalPercentualReceita.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }
  
  get gridColsReturnValue(): number {
    return this.gridPlateIsVisible ? 5 : 12;
  }
  
  public visibleTpCard() {
    const tpReceitaCard = Metadata.getInstance<Select>('TPRECEITA');
    const tpReceita = Metadata.getInstance<Select>('TPRECEITA1');
    
    const cardapio = Metadata.getInstance<Select>('TIPOCARD_ORIGEM');
    const central = Metadata.getInstance<Select>('CENTRAL');
    central.disabled = true;

    if (this.utilitiesPLA.isMerenda()) {
      tpReceitaCard.isVisible = true;
      tpReceita.isVisible = false;
      
      central.value = this.appController.getCdCentral();
    } else {
      tpReceitaCard.isVisible = false;
      tpReceita.isVisible = true;
      cardapio.isVisible = false;
      central.isVisible = false;
    }
  }
  
  public changeRecipe({ component }: IEventParam<Select>) {
    const unidadeO = Metadata.getInstance<Select>('UNIDADE_ORIGEM');
    const tipoCardO = Metadata.getInstance<Select>('TIPOCARD_ORIGEM');
    const central = Metadata.getInstance<Select>('CENTRAL');
    const servicoO = Metadata.getInstance<Select>('SERVICO_ORIGEM');
    
    if (component.value === 'T') {
      unidadeO.disabled = true;
      unidadeO.removeValidation('required');
      unidadeO.setValue('');
      servicoO.disabled = true;
      servicoO.removeValidation('required');
      servicoO.setValue('');
      tipoCardO.addValidation('required', { message: 'Campo obrigatório' });
      tipoCardO.disabled = false;
      central.addValidation('required', { message: 'Campo obrigatório' });
      central.disabled = !this.prato.permiteSelecCentral;
    } else if (component.value === 'L') {
      unidadeO.disabled = false;
      unidadeO.addValidation('required', { message: 'Campo obrigatório' });
      unidadeO.setValue('');
      servicoO.disabled = true;
      servicoO.removeValidation('required');
      servicoO.setValue('');
      tipoCardO.disabled = true;
      tipoCardO.removeValidation('required');
      tipoCardO.setValue('');
      central.disabled = true;
      central.removeValidation('required');
    } else if (component.value === 'S') {
      unidadeO.disabled = false;
      servicoO.disabled = false;
      tipoCardO.disabled = true;
      central.disabled = true;
      unidadeO.addValidation('required', { message: 'Campo obrigatório' });
      servicoO.addValidation('required', { message: 'Campo obrigatório' });
      tipoCardO.removeValidation('required');
      tipoCardO.setValue('');
      central.removeValidation('required');
      // central.setValue('');
    } else { // P
      unidadeO.removeValidation('required');
      servicoO.removeValidation('required');
      tipoCardO.removeValidation('required');
      central.removeValidation('required');
      unidadeO.disabled = true;
      servicoO.disabled = true;
      tipoCardO.disabled = true;
      central.disabled = true;
      unidadeO.setValue('');
      servicoO.setValue('');
      tipoCardO.setValue('');
      // central.setValue('');
    }
    this.carregaPratos();
  }
  
  public setFilterService({ component }: IEventParam<Select>) {
    const form: Form = Metadata.getInstance('formOrigemPercentServ');
    const row = form.value;
    const conditions = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: row.UNIDADE_ORIGEM,
      },
      {
        name: 'IDTIPOSERV',
        operator: '=',
        value: this.utilitiesPLA.isMerenda() ? '' : 'N',
      },
    ];
    component.datasource.filter = conditions;
    component.datasource.get();
  }
  
  public changeUnityServ() {
    // const grid: Grid = Metadata.getInstance('gridPrato');
    const form: Form = Metadata.getInstance('formOrigemPercentServ');
    const row = form.value;
    
    if (row.TPRECEITA === 'S' || row.TPRECEITA1 === 'S') {
      const serv = Metadata.getInstance<Select>('SERVICO_ORIGEM');
      serv.setValue('');
      // grid.datasource.data = [];
    }
    this.carregaPratos();
  }
  
  public async carregaPratos() {
    LoadingService.show(' ');
    const formO: Form = Metadata.getInstance('formOrigemPercentServ');
    // const formServDest: Form = Metadata.getInstance('formPercentServ');
    const gridPratos: Grid = Metadata.getInstance('gridPratos');
    const gridPercentServ: GridEditable = Metadata.getInstance('gridPercentualReceitasSevico');
    const row = formO.value;
    
    if (this.shouldLoadPratos && formO.validate() && gridPercentServ.datasource.currentRow.CDSERVFILI) {
      // const central = this.appController.getCdCentral();
      const central = row.CENTRAL;
      const conditions = [
        {
          name: 'CDPRATO',
          operator: '=',
          value: row.PRATOINICIAL.CDPRATO,
        },
        {
          name: 'TIPO_REC',
          operator: '=',
          value: row.TPRECEITA !== null ? row.TPRECEITA : row.TPRECEITA1,
        },
        {
          name: 'CDSERVFILI',
          operator: '=',
          value: row.SERVICO_ORIGEM,
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: row.UNIDADE_ORIGEM,
        },
        {
          name: 'CDFILAUXPRAT',
          operator: '=',
          value: row.PRATOINICIAL.CDFILAUXPRAT,
        },
        {
          name: 'CDTIPOCARD',
          operator: '=',
          value: row.TIPOCARD_ORIGEM,
        },
        {
          name: 'CDFILCENTPME',
          operator: '=',
          value: row.TPRECEITA === 'T' ? central : '',
        },
        {
          name: 'VRPERCENTUAL',
          operator: '=',
          value: gridPercentServ.datasource.currentRow.PERCENTUAL,
        },
      ];
      
      gridPratos.datasource.filter = conditions;
      gridPratos.datasource.loading = true;
      const cloneConfig = gridPratos.datasource.clone();
      const cloneDs = DatasourceFactory.factory(cloneConfig);
      await cloneDs.get().then(() => {
        const perc = gridPercentServ.datasource.currentRow.PERCENTUAL;
        for (let i = 0; i < cloneDs.data.length; i += 1) {
          cloneDs.data[i].QUANTIDADE *= (perc / 100);
          cloneDs.data[i].QTREQUERIDA *= (perc / 100);
          cloneDs.data[i].QTNUTRI *= (perc / 100);
          cloneDs.data[i].QTBRUTA *= (perc / 100);
        }
        gridPratos.datasource.updateData(cloneDs.data);
        gridPratos.datasource = cloneDs;
        // this.cardPlatesVisible();
      }).finally(() => {
        LoadingService.hide();
        this.shouldLoadPratos = false;
      });
    } else {
      // gridPercentServ.datasource.data = [];
      gridPratos.datasource.data = [];
      this.closeCardPlates();
      LoadingService.hide();
    }
  }
  
  private cardPlatesVisible(column: any) {
    const card = Metadata.getInstance<Card>('pratosOrigemCard');
    const grid: GridEditable = Metadata.getInstance('gridPercentualReceitasSevico');
    const formO: Form = Metadata.getInstance('formOrigemPercentServ');
    try {
      const columnName = column.column.name;

      if (columnName !== 'PERCENTUAL' && grid.datasource.data.length > 0 && formO.validate() && !grid.editing) {
        card.isVisible = true;
        this.gridPlateIsVisible = true;
        this.shouldLoadPratos = true;
      } else {
        card.isVisible = false;
        this.gridPlateIsVisible = false;
      }
    } catch (error) {
      card.isVisible = false;
      this.gridPlateIsVisible = false;
    }
  }
  
  public closeCardPlates() {
    const card = Metadata.getInstance<Card>('pratosOrigemCard');
    const gridPlates: GridEditable = Metadata.getInstance('gridPratos');
    this.gridEditableController.cancelEditing({ component: gridPlates });
    
    card.isVisible = false;
    this.gridPlateIsVisible = false;
  }
  
  public setGridPercentService() {
    const form: Form = Metadata.getInstance('formPercentServ');
    // const formO: Form = Metadata.getInstance('formOrigemPercentServ');
    const grid: GridEditable = Metadata.getInstance('gridPercentualReceitasSevico');
    const row = form.value;
    
    if (form.validate()) {
      const conditions = [
        {
          name: 'CDGRUPSERV',
          operator: '=',
          value: row.GRUPOSERVICO !== null ? row.GRUPOSERVICO : 'T',
        },
        {
          name: 'CDFILIAL',
          operator: '=',
          value: row.UNIDADE,
        },
        {
          name: 'IDTIPOSERV',
          operator: '=',
          value: this.utilitiesPLA.isMerenda() ? '' : 'N',
        },
      ];
      grid.datasource.filter = conditions;
      grid.datasource.get();
      // this.carregaPratos();
    } else grid.datasource.data = [];
  }
  
  public replicacao() {
    const gridS: GridEditable = Metadata.getInstance('gridPercentualReceitasSevico');
    // const gridP: Grid = Metadata.getInstance('gridPratos');
    const formO: Form = Metadata.getInstance('formOrigemPercentServ');
    const formS: Form = Metadata.getInstance('formPercentServ');
    const checkedRows = gridS.selectedRows;
    
    const rowO = formO.value;
    const rowS = formS.value;
    
    // rowO.CDFILCENTPME = this.appController.getCdCentral();
    rowO.CDFILCENTPME = formO.value.CENTRAL;
    
    if (checkedRows.length > 0) {
      LoadingService.show(' ');
      Http.post('/precentReceita/replicacao', { rowO, rowS, checkedRows }).then((response) => {
        if (response.data) {
          if (response.data.dataset) {
            if (response.data.dataset.RETORNO[0]) {
              this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
              this.closeCardPlates();
              gridS.reload();
            }
          } else this.utilities.showAlert('alert', `pme::${response.data.error}`, 'error');
        }
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        gridS.selectedRows = [];
        LoadingService.hide();
      });
    } else this.utilities.showAlert('alert', 'Nenhum registro selecionado.', 'warning');
  }
  
  public savePercServ() {
    const formO: Form = Metadata.getInstance('formOrigemPercentServ');
    const formServDest: Form = Metadata.getInstance('formPercentServ');
    const grid: GridEditable = Metadata.getInstance('gridPercentualReceitasSevico');
    const editedRows = grid.getEditedRows(false, true);
    
    if (formO.validate() && formServDest.validate() && grid.isGridValid(true)) {
      if (editedRows.length > 0) {
        this.utilities.showAlert('alert', 'Salvo com sucesso!', 'success');
        this.atualizarGridPercServ();
      } else {
        this.gridEditableController.cancelEditing({ component: grid });
        this.utilities.showAlert('alert', 'Nenhum registro editado.', 'error');
      }
    }
  }
  
  public atualizarGridPercServ() {
    const grid: GridEditable = Metadata.getInstance('gridPercentualReceitasSevico');
    const editedRows = grid.getEditedRows() as IDictionary;
    const data = grid.datasource.data;
    this.gridEditableController.defaultState(grid, true).then(() => {
      grid.datasource.data.forEach((row: IDictionary) => {
        const foundIndex = grid.datasource.data.findIndex((serv: IDictionary) => serv.CDSERVFILI === row.CDSERVFILI);
        if (foundIndex >= 0) grid.datasource.data[foundIndex].PERCENTUAL = data[foundIndex].PERCENTUAL;
      });
      editedRows.forEach((editedRow: IDictionary) => {
        const foundIndex = grid.datasource.data.findIndex((serv: IDictionary) => serv.CDSERVFILI === editedRow.CDSERVFILI);
        grid.datasource.data[foundIndex].PERCENTUAL = editedRow.PERCENTUAL;
      });
      grid.datasource.currentRow = {};
      this.carregaPratos();
    });
  }
  
  public cancelEditing({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    if (grid.editing) {
      const data = grid.datasource.data;
      this.gridEditableController.defaultState(grid).then(() => {
        grid.datasource.data.forEach((row: IDictionary) => {
          const foundIndex = grid.datasource.data.findIndex((serv: IDictionary) => serv.CDSERVFILI === row.CDSERVFILI);
          if (foundIndex >= 0) grid.datasource.data[foundIndex].PERCENTUAL = data[foundIndex].PERCENTUAL;
        });
        grid.datasource.currentRow = {};
        this.carregaPratos();
      });
    }
  }
  
  public reload({ component }: IDictionary) {
    const grid = this.utilities.returnGrid(component) as GridEditable;
    const data = grid.datasource.data;
    grid.datasource.get().then(() => {
      if (grid.editing) {
        this.gridEditableController.defaultState(grid).then(() => {
          grid.datasource.data.forEach((row: IDictionary) => {
            const foundIndex = grid.datasource.data.findIndex((serv: IDictionary) => serv.CDSERVFILI === row.CDSERVFILI);
            if (foundIndex >= 0) grid.datasource.data[foundIndex].PERCENTUAL = data[foundIndex].PERCENTUAL;
          });
          grid.datasource.currentRow = {};
          this.carregaPratos();
        });
      } else {
        grid.datasource.data.forEach((row: IDictionary) => {
          const foundIndex = grid.datasource.data.findIndex((serv: IDictionary) => serv.CDSERVFILI === row.CDSERVFILI);
          if (foundIndex >= 0) grid.datasource.data[foundIndex].PERCENTUAL = data[foundIndex].PERCENTUAL;
        });
        grid.datasource.currentRow = {};
        this.carregaPratos();
      }
    });
  }
}
