import {
  Loader,
  Metadata,
  IDictionary,
  Http,
} from '@zeedhi/core';

import {
  GridEditable, Form, DialogService, Button, LoadingService,
} from '@zeedhi/common';
  
export class Ger30002PadraoDeServicoController {
  public gridEditController = Loader.getInstance('GridEditableController');
  
  public modalController = Loader.getInstance('ModalController');

  public crudController = Loader.getInstance('CrudController');

  public utilities = Loader.getInstance('Utilities');
  
  public appController = Loader.getInstance('AppController');
  
  public confirmaAcaoAtualizacao() {
    const gridPadraoServico: GridEditable = Metadata.getInstance('gridPadraoDeServico');
    const numLinhasSelecionadasGrid = gridPadraoServico.selectedRows.length;
    DialogService.show({
      name: 'confirmarAcaoAtualizacaoSimOuNao',
      type: 'warning',
      title: 'Atenção',
      text: (numLinhasSelecionadasGrid > 1)
        ? 'Deseja atualizar os serviços correspondentes aos Padrões de Serviços selecionados?'
        : 'Deseja atualizar os serviços correspondentes ao Padrão de Serviço selecionado?',
      buttons: [
        {
          name: 'progressLine',
          component: 'ZdProgress',
          label: 'Progress',
          indeterminate: true,
          cssClass: 'loading-progress-line',
        },
        {
          name: 'yesDialog',
          component: 'ZdButton',
          label: 'Sim',
          events: { click: () => this.realizaAcaoAtualizacao(gridPadraoServico) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: 'Não',
          events: { click: () => this.hideDialog() },
        },
      ],
    });
  }
  
  public hideDialog() {
    const buttonYes = Metadata.getInstance<Button>('yesDialog');
    this.utilities.disableLoadingButton(buttonYes);
    DialogService.hide();
  }
  
  private realizaAcaoAtualizacao(gridPadraoServico: GridEditable) {
    // const buttonYes = Metadata.getInstance<Button>('yesDialog');
    const padroesServicosSelecionados = gridPadraoServico.selectedRows;
    const formModalPadraoServico: Form = Metadata.getInstance('formModalPadraoServico');
    let unidadesSelecionadasForm: IDictionary = [];
    if (formModalPadraoServico.value.UNIDADE.length > 0) {
      unidadesSelecionadasForm = formModalPadraoServico.value.UNIDADE;
    } else {
      unidadesSelecionadasForm = [];
    }
    LoadingService.show(' ');
    Http.post('/updateAtualizacaoAutomatica', { padroesServicosSelecionados, unidadesSelecionadasForm }).then(() => {
      this.utilities.showAlert('alert', 'Atualizado com sucesso!', 'success');
      DialogService.hide();
      this.modalController.closeModal();
    }).catch((error: string) => {
      this.utilities.showAlert('alertError', `pme::${error}`, 'error');
      DialogService.hide();
      this.modalController.closeModal();
    }).finally(() => {
      LoadingService.hide();
    });
  }
}
