import {
  Modal, ModalService, Button,
  Grid, Select,
  IButtonEvent, Date as ZdDate, Radio,
  Form, LoadingService,
} from '@zeedhi/common';
import {
  Metadata, IDictionary, IEventParam,
  Loader, Http, dayjs as moment,
} from '@zeedhi/core';
import { Cardapio } from '../../../components/cardapio/common/Cardapio';

export class Pme00602ImportarCardDia {
  private modalImportCard!: Modal;
  
  private modalImportacao!: Modal;
  
  private elaboracaoCardapio = Loader.getInstance('Pme00600ElaboracaoCardapio');
  
  public utilities = Loader.getInstance('Utilities');
  
  public colsGridPara = 12;
  
  public showGridPrato = false;
  
  public showArredondar = false;
  
  public NRCARDPMEORIGEM = '';
  
  private datasourceDePara: IDictionary[] = [];
  
  // Funçoes search modal Importacao
  public async createPeriodoModal() {
    this.modalImportacao = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/cardapio/modals/gridPeriodoImportCard' });
  }
  
  public openModalImportacao() {
    this.modalImportacao.show();
  }
  
  public destroyPeriodoModal() {
    this.modalImportacao.destroy();
  }
  
  public closePeriodoModal() {
    this.modalImportacao.hide();
  }
  // ---------
  
  public async createModal() {
    this.modalImportCard = await ModalService.createFromJsonPost('',
      { jsonPath: 'pme/cardapio/modals/pme00602_importar_cardapio_dia' });
  }
  
  public fullscreen({ component }: IButtonEvent) {
    const modal: IDictionary = Metadata.getInstance('modalHeaderImportarCardDia');
    modal.parent.fullscreen = !modal.parent.fullscreen;
    component.iconName = (modal.parent.fullscreen) ? 'mdi-fullscreen-exit' : 'mdi-fullscreen';
  }
  
  public destroyModal() {
    if (this.modalImportCard) {
      this.modalImportCard.destroy();
      this.modalImportCard = this.utilities.modalEmpty;
      this.closePeriodoModal();
      this.colsGridPara = 12;
      this.showGridPrato = false;
      this.NRCARDPMEORIGEM = '';
      this.datasourceDePara = [];
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.setValueAction(false);
    }
  }
  
  public cancelModal() {
    if (this.modalImportCard) {
      this.modalImportCard.hide();
      this.closePeriodoModal();
      this.colsGridPara = 12;
      this.showGridPrato = false;
      this.NRCARDPMEORIGEM = '';
      this.datasourceDePara = [];
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.setValueAction(false);
    }
  }
  
  public showModal({ event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    this.validAndOpenModal();
  }
  
  public async validAndOpenModal() {
    if (this.elaboracaoCardapio.getFilterData.NRCARDPME) {
      this.cancelModal();
      const cardapio: Cardapio = Metadata.getInstance('cardapio');
      cardapio.setValueAction(true);
      if (this.utilities.checkOpenModalOverlap()) {
        if (!this.modalImportCard) {
          this.createModal().then(() => {
            this.modalImportCard.show();
          });
        } else {
          this.modalImportCard.show();
        }
        this.createPeriodoModal();
      }
    } else {
      this.utilities.showAlert('withoutFilter', 'Operação bloqueada! Favor realizar o filtro.', 'error');
    }
  }
  
  public clearFields({ component }: IEventParam<Select>) {
    const cardapio = Metadata.getInstance<Select>('TipoCardImportCarDia');
    const servico = Metadata.getInstance<Select>('servicoImportCarDia');
    const gridDePara = Metadata.getInstance<Grid>('gridDePara');
    const gridPrato = Metadata.getInstance<Grid>('gridPrato');
    const periodoDe = Metadata.getInstance<ZdDate>('PeriodoDeCarDia');
    const periodoAte = Metadata.getInstance<ZdDate>('PeriodoAteCarDia');
    const dataDe = Metadata.getInstance<ZdDate>('DtDeCarDia');
    const dataAte = Metadata.getInstance<ZdDate>('DtAteCarDia');
    
    if (component.name === 'centralImportCarDia') cardapio.setValue('');
    
    servico.setValue('');
    periodoDe.value = '';
    periodoAte.value = '';
    dataDe.value = '';
    dataAte.value = '';
    gridDePara.datasource.data = [];
    gridPrato.datasource.data = [];
  }
  
  public setFilterTpCardapio({ component }: IEventParam<Select>) {
    const form = Metadata.getInstance<Form>('formCardDia');
    const filter: IDictionary = [];
    this.utilities.onMountedFilterFind(filter, 'CDFILCENTPME', '=', form.value.centralImportCarDia);
    this.utilities.onMountedFilterFind(filter, 'NRPASTATCARD', '=', null);
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public hasShowArredondar() {
    const IDPERCCOME = (localStorage.getItem('IDPERCCOME') === null) ? 'C' : localStorage.getItem('IDPERCCOME');
    this.showArredondar = (IDPERCCOME === 'C');
  }
  
  public setFilterServ({ component }: IEventParam<Select>) {
    const formCardDia = Metadata.getInstance<Form>('formCardDia');
    const form2 = Metadata.getInstance<Form>('form2');
    const filter = [
      {
        name: 'CDTIPOCARD',
        operator: '=',
        value: form2.value.TipoCardImportCarDia ? form2.value.TipoCardImportCarDia : '',
      },
      {
        name: 'CDFILCENTPME',
        operator: '=',
        value: formCardDia.value.centralImportCarDia,
      },
    ];
    
    component.datasource.filter = filter;
    component.datasource.get();
  }
  
  public openModalSearch() {
    const form = Metadata.getInstance<Form>('form2');
    const tipoCard = form.value.TipoCardImportCarDia;
    const central = Metadata.getInstance<Form>('formCardDia').value.centralImportCarDia;
    if (tipoCard) {
      this.openModalImportacao();
      this.utilities.setCentral(central);
      this.utilities.setCdTipoCard(tipoCard);
      this.utilities.setFilterModalPeriodoImport();
    } else {
      this.utilities.showAlert('alertError', 'Tipo de Cardápio não selecionado', 'warning');
    }
  }
  
  public reloadPeriodoModal() {
    const central = Metadata.getInstance<Form>('formCardDia').value.centralImportCarDia;
    const tipoCard = Metadata.getInstance<Form>('form2').value.TipoCardImportCarDia;
    this.utilities.setFilterModalPeriodoCentralModal(central, tipoCard);
  }
  
  public setPeriodoParams({ row }: IDictionary) {
    const periodoDe = Metadata.getInstance<ZdDate>('PeriodoDeCarDia');
    const periodoAte = Metadata.getInstance<ZdDate>('PeriodoAteCarDia');
    const dtDe = Metadata.getInstance<ZdDate>('DtDeCarDia');
    const dtAte = Metadata.getInstance<ZdDate>('DtAteCarDia');
    const dtIni = moment(row.DTINICARD, 'DD/MM/YYYY');
    const dtFin = moment(row.DTFINCARD, 'DD/MM/YYYY');
    periodoDe.value = dtIni.format('DD/MM/YYYY');
    periodoAte.value = dtFin.format('DD/MM/YYYY');
    dtDe.value = dtIni.format('DD/MM/YYYY');
    dtAte.value = dtFin.format('DD/MM/YYYY');
    this.NRCARDPMEORIGEM = row.NRCARDPME;
    this.modalImportacao.hide();
    this.filterGridPara();
  }
  
  public setDataPosicionada() {
    const dtIncluir = Metadata.getInstance<ZdDate>('DTINCLUIR');
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    dtIncluir.value = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME.substring(0, 10);
  }
  
  public setRadioDefault({ component }: IEventParam<Radio>) {
    component.value = (component.datasource as IDictionary).data[0].value;
  }
  
  // Quando mudar as funções de data para change habilitar essa função novamente;
  // public setDefaultInclude({ component }: IEventParam<ZdDate>) {
  //   const cardapio: Cardapio = Metadata.getInstance('cardapio');
  //   component.value = cardapio.getCurrentSelectedRegister()?.DTCARDIAPME.substring(0, 10);
  // }
  
  public filterGridPara() {
    const formCardDia = Metadata.getInstance<Form>('formCardDia').validate();
    const form2 = Metadata.getInstance<Form>('form2').validate();
    const form3 = Metadata.getInstance<Form>('form3').validate();
    // const gridDePara = Metadata.getInstance<Grid>('gridDePara');
    
    if (formCardDia && form2 && form3) {
      const filter: IDictionary = [];
      const filial = (this.elaboracaoCardapio.getFilterData.aggregated?.CDFILIAL === undefined)
        ? null
        : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
      this.utilities.onMountedFilterFind(filter, 'P_NRCARDPME', '=', this.NRCARDPMEORIGEM);
      this.utilities.onMountedFilterFind(
        filter, 'P_CDFILCENTPME', '=',
        Metadata.getInstance<Select>('centralImportCarDia').value,
      );
      this.utilities.onMountedFilterFind(
        filter, 'P_CDSERVFILI', '=',
        Metadata.getInstance<Select>('servicoImportCarDia').value,
      );
      this.utilities.onMountedFilterFind(filter, 'P_CDFILAGREG', '=', filial);
      this.utilities.onMountedFilterFind(filter, 'P_DTINICIAL', '=', Metadata.getInstance<ZdDate>('PeriodoDeCarDia').value);
      this.utilities.onMountedFilterFind(filter, 'P_DTFINAL', '=', Metadata.getInstance<ZdDate>('PeriodoAteCarDia').value);
      
      LoadingService.show(' ');
      Http.post('/find/datasDePara', { requestType: 'FilterData', filter, limit: 1000000 }).then((dePara) => {
        this.mountDataSourceDePara(dePara.data.data);
      });
    }
  }
  
  private mountDataSourceDePara(dePara: IDictionary) {
    this.datasourceDePara = [];
    const dtDe = Metadata.getInstance<ZdDate>('DtDeCarDia');
    const dtAte = Metadata.getInstance<ZdDate>('DtAteCarDia');
    const incluirCardDe = Metadata.getInstance<ZdDate>('DTINCLUIR');
    
    const idxInicial = dePara.findIndex((data: IDictionary) => data.DTCARDIAPME.substring(0, 10) === dtDe.value);
    const idxFinal = dePara.findIndex((data: IDictionary) => data.DTCARDIAPME.substring(0, 10) === dtAte.value);
    
    const newDePara = dePara.slice(idxInicial, (idxFinal + 1));
    
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const idxInicialCard = cardapio.dates.findIndex(
      (data: IDictionary) => data.DTCARDIAPME.substring(0, 10) === incluirCardDe.value,
    );
    const newDateCard = cardapio.dates.slice(idxInicialCard, cardapio.dates.length);
    
    newDateCard.forEach((object: IDictionary, idx: number) => {
      if (newDePara.length > idx) {
        const template: IDictionary = {};
        template.DE = moment(newDePara[idx].DTCARDIAPME, 'DD/MM/YYYY').format(this.getFormatLocaleFullWeek());
        template.PARA = moment(object.DTCARDIAPME, 'DD/MM/YYYY').format(this.getFormatLocaleFullWeek());
        this.datasourceDePara.push(template);
      }
    });
    
    if (this.datasourceDePara.length > 0) {
      this.requestCorDePara(this.datasourceDePara).then((corDePara) => {
        if (corDePara.length > 0) this.attributeColor(corDePara, this.datasourceDePara);
        const gridDePara = Metadata.getInstance<Grid>('gridDePara');
        // gridDePara.datasource.loading = true;
        gridDePara.datasource.updateData(this.datasourceDePara).then((data: IDictionary) => {
          gridDePara.datasource.currentRow = data[0];
          if (!this.showGridPrato) this.changeVisibilityGridPrato();
          this.filterGridPratoCardDia({ row: data[0] });
          LoadingService.hide();
        });
      });
    }
  }
  
  // PEGA A COR DAS DATAS DE-PARA
  private requestCorDePara(datasource: IDictionary) {
    const filter: IDictionary = [];
    const filial = (this.elaboracaoCardapio.getFilterData.aggregated?.CDFILIAL === undefined)
      ? Metadata.getInstance<Select>('centralImportCarDia').value
      : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
    this.utilities.onMountedFilterFind(filter, 'P_CDFILIAL', '=', filial);
    this.utilities.onMountedFilterFind(
      filter, 'P_DTINIORIG', '=',
      moment(datasource[0].DE.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY'),
    );
    this.utilities.onMountedFilterFind(
      filter, 'P_DTFIMORIG', '=',
      moment(datasource[datasource.length - 1].DE.substring(0, 10),
        this.utilities.getFormatLocale()).format('DD/MM/YYYY'),
    );
    this.utilities.onMountedFilterFind(
      filter, 'P_DTINIDEST', '=',
      moment(datasource[0].PARA.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY'),
    );
    this.utilities.onMountedFilterFind(
      filter, 'P_DTFIMDEST', '=',
      moment(datasource[datasource.length - 1].PARA.substring(0, 10),
        this.utilities.getFormatLocale()).format('DD/MM/YYYY'),
    );
    
    return Http.post('/find/corDePara', { requestType: 'FilterData', filter, limit: 1000000 }).then((corDePara) => {
      const data = corDePara.data.data;
      return data;
    });
  }
  
  public filterGridPratoCardDia({ row }: IDictionary) {
    const gridPrato = Metadata.getInstance<Grid>('gridPrato');
    const filter: IDictionary = [];
    // const filial = (this.elaboracaoCardapio.getFilterData.aggregated?.CDFILIAL === undefined) ?
    // Metadata.getInstance<Select>('centralImportCarDia').value : this.elaboracaoCardapio.getFilterData.aggregated.CDFILIAL;
    this.utilities.onMountedFilterFind(filter, 'P_NRCARDPME', '=', this.NRCARDPMEORIGEM);
    this.utilities.onMountedFilterFind(
      filter, 'P_CDFILCENTPME', '=', Metadata.getInstance<Select>('centralImportCarDia').value,
    );
    this.utilities.onMountedFilterFind(
      filter, 'P_CDSERVFILI', '=', Metadata.getInstance<Select>('servicoImportCarDia').value,
    );
    // this.utilities.onMountedFilterFind(filter, 'P_CDFILAGREG', '=', filial);
    this.utilities.onMountedFilterFind(
      filter, 'P_CDFILAGREG', '=', Metadata.getInstance<Select>('centralImportCarDia').value,
    );
    this.utilities.onMountedFilterFind(
      filter, 'P_DTCARDIAPME', '=',
      moment(row.DE.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY'),
    );
    
    gridPrato.datasource.filter = filter;
    gridPrato.datasource.get();
  }
  
  // ATRIBUI A COLORAÇÃO DE ACORDO COM OQUE VEM DO BANCO.
  private attributeColor(corDePara: IDictionary, datasource: IDictionary) {
    corDePara.forEach((cor: IDictionary) => {
      const hasData = datasource.find((data: IDictionary) => cor.DTFERIFILI.substring(0, 10) === data.DE.substring(0, 10));
      if (hasData) {
        hasData.IDCARDIAUTIL = cor.IDCARDIAUTIL;
        hasData.NRCORNUTIL = cor.NRCORNUTIL;
      }
    });
  }
  
  public coloredText({ row }: IDictionary) {
    if (row.IDCARDIAUTIL === undefined || row.IDCARDIAUTIL === 'N') return '';
    
    let color = row.NRCORNUTIL;
    color = this.utilities.TColorToHex(color);
    color = color.substring(1, color.length);
    if (color.length < 6) color = this.utilities.increamentZeroInHex(color);
    return `#${color}`;
  }
  
  public getFormatLocaleFullWeek(): string {
    return (this.utilities.getLanguage() !== 'en_us') ? 'DD/MM/YYYY dddd' : 'MM/DD/YYYY dddd';
  }
  
  private changeVisibilityGridPrato() {
    this.showGridPrato = !this.showGridPrato;
    this.colsGridPara = (this.showGridPrato) ? 6 : 12;
  }
  
  // IMPORTAÇÃO DO CARDAPIO
  public importarCardapio() {
    const formCardDia = Metadata.getInstance<Form>('formCardDia').validate();
    const form2 = Metadata.getInstance<Form>('form2').validate();
    const form3 = Metadata.getInstance<Form>('form3').validate();
    
    if (formCardDia && form2 && form3) {
      if (this.datasourceDePara.length > 0) {
        const row: IDictionary = {};
        row.formImport = this.mountFormImport();
        row.filterData = this.elaboracaoCardapio.getFilterData;
        row.dePara = this.mountDeParaImport();
        LoadingService.show(' ');
        Http.post('/importarCardapio', { requestType: 'Row', row }).then((response) => {
          if (response.data.dataset.successImport[0]) {
            this.destroyModal();
            this.utilities.showAlert('successImport', 'Cardápio importado com sucesso.', 'success');
            this.elaboracaoCardapio.reloadCard();
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', `pme::${error}`, 'error');
        }).finally(() => {
          LoadingService.hide();
        });
      } else {
        this.utilities.showAlert('cardapioOrigem', 'Não possui cardápio de origem.', 'error');
      }
    }
  }
  
  private mountFormImport() {
    const form3 = Metadata.getInstance<Form>('form3');
    return {
      CENTRAL: Metadata.getInstance<Form>('formCardDia').value.centralImportCarDia,
      TIPOCARD: Metadata.getInstance<Form>('form2').value.TipoCardImportCarDia,
      SERVICO: form3.value.servicoImportCarDia,
      NRCARDPME: this.NRCARDPMEORIGEM,
      ARREDONDAR: form3.value.arredondarComen,
      RADIOSELECT: form3.value.radioSelect,
    };
  }
  
  // VERIFICA SE AS DATAS QUE IRAM SER IMPORTADAS SÃO FERIADO OU BLOQUEADAS. PARA PODER ENVIAR APENAS AS DATAS CORRETAS.
  private mountDeParaImport() {
    const dePara: IDictionary[] = [];
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    
    this.datasourceDePara.forEach((data: IDictionary) => {
      const template: IDictionary = {};
      const dataPara = moment(data.PARA.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY');
      const dataCard = cardapio.dates.find(
        (dataCardapio: IDictionary) => dataCardapio.DTCARDIAPME.substring(0, 10) === dataPara,
      );
      if (dataCard.DTFERIFILI == null && cardapio.dateLessDateOfProcess(dataCard.DTCARDIAPME)) {
        template.DE = moment(data.DE.substring(0, 10), this.utilities.getFormatLocale()).format('DD/MM/YYYY');
        template.PARA = dataPara;
        dePara.push(template);
      }
    });
    
    return dePara;
  }
  
  public validData() {
    const format = 'DD/MM/YYYY';
    const periodoDe = moment(Metadata.getInstance<ZdDate>('PeriodoDeCarDia').value, format);
    const periodoAte = moment(Metadata.getInstance<ZdDate>('PeriodoAteCarDia').value, format);
    const dtDe = moment(Metadata.getInstance<ZdDate>('DtDeCarDia').value, format);
    const dtAte = moment(Metadata.getInstance<ZdDate>('DtAteCarDia').value, format);
    
    if ((dtDe < periodoDe || dtDe > periodoAte) || (dtAte < dtDe || dtAte > periodoAte)) {
      this.utilities.showAlert('foraDoPeriodo', 'Data fora do período selecionado.', 'error');
      this.resetAllGrid();
    } else {
      this.filterGridPara();
    }
  }
  
  public validaDataIncluir({ component }: IEventParam<ZdDate>): boolean {
    const cardapio: Cardapio = Metadata.getInstance('cardapio');
    const hasDtIncluir = cardapio.dates.find((data: IDictionary) => data.DTCARDIAPME.substring(0, 10) === component.value);
    
    if (hasDtIncluir === undefined) {
      this.utilities.showAlert('foraDoPeriodo', 'Data de Inclusão fora do período do cardápio.', 'error');
      this.resetAllGrid();
    } else {
      this.filterGridPara();
    }
    
    return true;
  }
  
  private resetAllGrid() {
    this.datasourceDePara = [];
    const gridDePara = Metadata.getInstance<Grid>('gridDePara');
    gridDePara.datasource.updateData(this.datasourceDePara);
    this.changeVisibilityGridPrato();
  }
}
